% Author: Mehmet Deniz Cicek
% Date  : 06/30/2010
% 
% The following code illustrates how to call the function
% scaled_and_shifted_kw(...) to solve a stochastic maximization problem.
% The function is myfnc(x, x_true, a1, sig) which returns a noisy 
% observation of the multidimensional function -a1*||x-x_true||^2. 
% In other words myfnc_multidim(x,x_true,a1,sig)= ...
%   - a1* sum((x - x_true).^2) + sig*randn(1,length(x)).

% Clean-up the workspace.
clc; close all; clear;

% Set the mandatory parameters of the ScaledAndShiftedKW
% function:
x_init = [30;30];
b_up   = [50;50];
b_dn   = -b_up;

% The following parameters are passed into the function and are 
% parameters of the function to be maximized.
x_true = [0.0;0.0];
a1=0.001;
sig=0.01;

% Set the options: 
% option1 is for the truncated KW algorithm 
% option2 is for the scaled-and-shifted-KW algorithm using
% the default parameter setting for the rest of parameters
m=1000; % number of iterations
n=500;  % number of sample paths
out_ind=[1:m]; % return every iterate of the algorithm
options1=struct('m',m,'n',n,'out_ind',out_ind,...
    'a_numsc',0,'a_maxnumsh',0,'c_maxnumsc',0);
options2=struct('m',m,'n',n,'out_ind',out_ind);

x_true=x_true*ones(1,n);

% Set the initial seed for random numbers for replicability and call the
% scaled_and_shifted_kw function:
randn('state', 0);
[x_matrix1, oscper1, a_totsh1, a_totsc1, c_totsc1, a_numsh1, c_numsc1, lastadapt1, samplepath1] = ...
    sskw(@myfnc_multidim, x_init, b_dn, b_up ,options1, x_true, a1, sig);
randn('state', 0);
[x_matrix2, oscper2, a_totsh2, a_totsc2, c_totsc2, a_numsh2, c_numsc2, lastadapt2, samplepath2] = ...
    sskw(@myfnc_multidim, x_init, b_dn, b_up ,options2, x_true, a1, sig);

x_true=permute(repmat(x_true,[1,1,m]),[3 1 2]);
% Calculate the MSE 
MSE_TruncatedKW=mean(squeeze(sum((x_matrix1-x_true).^2,2)),2);
MSE_ScaledAndShiftedKW=mean(squeeze(sum((x_matrix2-x_true).^2,2)),2);

%  Figure Plots
% MSE convergence plot on a log-log scale
figure(1)
loglog([1:m]',MSE_TruncatedKW,'r--');
hold on
loglog([1:m]',MSE_ScaledAndShiftedKW);
ylabel('MSE')
xlabel('Iteration n')
legend('TKW','SSKW')
title('MSE of the algorithms')

% end of ScaledAndShiftedKW_SampleCall.m