% Author: Mehmet Deniz Cicek
% Date  : 02/02/2009
% 
% The following code illustrates how to call the function
% scaled_and_shifted_kw(...) to solve a stochastic maximization problem.
% The function is myfnc(x, x_true, a1, sig) which returns a noisy 
% observation of -a1*(x-x_true)^2. In other words
% myfnc(x,x_true,a1,sig)= - a1* (x - x_true).^2 + sig*randn(1,length(x)).

% Clean-up the workspace.
clc; close all; clear;

% Set the mandatory parameters of the ScaledAndShiftedKW
% function:
x_init = 30;
b_up   = 50;
b_dn   = -b_up;

% The following parameters are passed into the function and are 
% parameters of the function to be maximized.
x_true = 0.0;
a1=0.001;
sig=0.01;

% Set the options: 
% option1 is for the truncated KW algorithm 
% option2 is for the scaled-and-shifted-KW algorithm using
% the default parameter setting for the rest of parameters
m=1000; % number of iterations
n=5000;  % number of sample paths
out_ind=[1:m]; % return every iterate of the algorithm
options1=struct('m',m,'n',n,'out_ind',out_ind,...
    'a_numsc',0,'a_maxnumsh',0,'c_maxnumsc',0);
options2=struct('m',m,'n',n,'out_ind',out_ind);

% Set the initial seed for random numbers for replicability and call the
% scaled_and_shifted_kw function:
randn('state', 0);
[x_matrix1, oscper1, a_totsh1, a_totsc1, c_totsc1, a_numsh1, c_numsc1, lastadapt1, samplepath1] = ...
    sskw(@myfnc, x_init, b_dn, b_up ,options1, x_true, a1, sig);
randn('state', 0);
[x_matrix2, oscper2, a_totsh2, a_totsc2, c_totsc2, a_numsh2, c_numsc2, lastadapt2, samplepath2] = ...
    sskw(@myfnc, x_init, b_dn, b_up ,options2, x_true, a1, sig);

% Calculate the final MSE 
Final_MSE_TruncatedKW=mean((x_matrix1(m,:)-x_true).^2)
Final_MSE_ScaledAndShiftedKW=mean((x_matrix2(m,:)-x_true).^2)

%  Figure Plots
% Sample path plot for the truncated KW algorithm
figure(1)
plot(x_matrix1(:,1)) % plots the first sample path
ylabel('X_n')
xlabel('Iteration n')
title('Truncated KW algorithm sample path')

% Sample path plot for the scaled-and-shifted KW algorithm
figure(2)
plot(x_matrix2(:,1)) % plots the first sample path
ylabel('X_n')
xlabel('Iteration n')
title('Scaled-and-shifted KW algorithm Sample Path')

% Relative frequency histogram of last iterate for truncated KW algorithm
figure(3)
[n1,xout1] = hist(x_matrix1(:,m),40);
bar(xout1,n1/n)
ylabel('Relative Frequency')
xlabel('Last iterate')
title('Truncated KW algorithm')

% Relative frequency histogram of last iterate for the scaled-and-shifted
% KW algorithm
figure(4)
[n2,xout2] = hist(x_matrix2(:,m),40);
bar(xout2,n2/n)
ylabel('Relative Frequency')
xlabel('Last iterate')
title('Scaled-and-shifted KW algorithm')

% end of ScaledAndShiftedKW_SampleCall.m