function [x_matrix, oscper, a_totsh, a_totsc, c_totsc, a_numsh, c_numsc, lastadapt, samplepath] = sskw(F, x_init, l, u, options, varargin);
% 
% Scaled-and-Shifted Kiefer-Wolfowitz Stochastic Approximation Algorithm:
% The algorithm is used to solve maximization problems for functions with
% noisy observations: 
%       max    F(x)
%       s.t.  l<=x<=u
% 
% INPUT PARAMETERS:
% F          function to maximize.  F is a string defining a function
%            of multiple variables, an inline function, a function handle, 
%            or a symbolic expression involving a multiple variable. 
%            Should accept matrix inputs with each column representing 
%            different point and if given a matrix x, to calculate the
%            function value at each column, should return a row vector 
%            of F(x) values. In order to use common random numbers in
%            gradient estimation, the user should define a global 
%            variable, CRN in the function F. Then, if the value of CRN
%            is zero, the state of the random number generator should be 
%            stored in another global variable, say RNDSTATE.
%            If CRN=1, then the state of the random number generator 
%            should be set to the value of RNDSTATE.
% x_init     vector of initial x values, if it is a single number, the
%            algorithm uses it for all sample paths; if it is a vector, the
%            given starting points are used for each sample path and the
%            number of samples is set to the size of x_init.
% l          lower boundary of x-values
% u          upper boundary of x-values
%
% OPTIONS:
% The following are optional parameters and can be defined using the 
% struct options. See examples below to see how to set these parameters.
% m            number of steps (default: 10,000)
% n            number of sample paths to generate (default: 15,000)
% theta_a,a    determines the sequence a(n) = theta_a x n^(-a)
%              (theta_a,a>0) (default: theta_a=1.0, a=1.0)
% theta_c,c    determines the sequence c(n) = theta_c x n^(-c)  
%              (theta_c,c>0) (default: theta_c=0.25, a=0.25)
% c_scl        the scale up factor for c(n) sequence (default:2.0)
% cmax         upper bound on the c(n) sequence; 
%              it should satisfy cmax<(u-l)/2 (default:0.2*(u-l))
% a_sh_bd      upper bound on the shift in the an sequence
%              (default: 10), the bound doubles everytime there's a shift
%              equal to the upper bound
% a_numsc      the number of scale-ups to the a(n) seq at the beginning of 
%              the alg so that the iterate hits the current truncation  
%              interval boundary (default: 2)
% a_sc_bd      upper bound on the amount of scale-up per iteration in the
%              a_n sequence (default:10)
% a_maxnumsh   upper bound on the number of shifts to the a(n) sequence
%              (default: 50)
% c_maxnumsc   upper bound on the number of scale-ups to the c(n) sequence
%              (default: 50)
% maxnumtrials upper bound on the number of iterations to force one
%              oscillation
% sampathno    the indices of paths to return all the details about; can
%              be a vector of indices, all from 1 to n
% out_ind      the iteration numbers for which the approximations are
%              returned as output (default: m)
% m_max        the iteration number after which no scaling and shifting is
%              allowed; should satisfy m_max>=a_numsc (default: m)
% varargin     arguments beyond the variable "options", e.g., 
%              [x_matrix ...] = mdim_sskw(F, x_init, l, u, options,p1,p2,...) 
%              are passed on to the function, F(x,p1,p2,..)
% 
% OUTPUT:
% x_matrix   returns the approximations at iteration numbers given in 
%            out_ind for each sample path, i.e., x_matrix(i,j,k) is the
%            j^th coordinate of out_ind(i)^th iterate 
%            on the k^th sample path. It is a 3D-matrix with size equal to 
%            [length of out_ind, dimension, number of sample paths n] 
% oscper     length of the oscillatory period for each sample path
% a_totsh    total shift in the a_n sequence in each sample path
% a_totsc    total scale-up in the a_n seq. in each sample path
% c_totsc    total scale-up in the c_n seq. in each sample path
% a_numsh    total number of shifts in the a_n seq. in each sample path
% c_numsc    total number of scale ups in the c_n seq. in each sample 
%            path
% lastadapt  the iteration number of the last scaling or shifting in  
%            each sample path 
% samplepath detailed sample paths for the paths with indices given by
%            'sampathno'. Each row includes the value of the gradient, the
%            value of the iterate, the value of the a_n sequence and the
%            value of the c_n sequence along each dimension at that
%            iteration. It is a matrix of size 
%            (m x 4*dimension x length(sampathno))
% 
% Example 1: Truncated KW algorithm (no scaling or shifting):
%            options=struct('a_numsc',0,'a_maxnumsh',0,'c_maxnumsc',0);
%            [x_matrix, oscper, a_totsh, a_totsc, c_totsc ,a_numsh, ...
%            c_numsc] = mdim_sskw(@F, x_init, l, u ,options);
% Example 2: To run a single path of the algorithm and return all 
%            iterates:
%            options=struct('n',1,'out_ind',[1:10000])
%            [x_matrix, oscper, a_totsh, a_totsc, c_totsc ,a_numsh, ...
%            c_numsc] = mdim_sskw(@F, x_init, l, u ,options);
% Example 3: To run 25,000 iterations for 1000 sample paths and return 
%            iterates at steps 5000, 10,000 and 25,000:
%            options=struct('m',25000,'out_ind',[5000 10000 25000])
%            [x_matrix, oscper, a_totsh, a_totsc, c_totsc ,a_numsh, ...
%            c_numsc] = mdim_sskw(@F, x_init, l, u ,options);
% Example 4: To run with all the defaults:
%            [x_matrix, oscper, a_totsh, a_totsc, c_totsc ,a_numsh, ...
%            c_numsc] = mdim_sskw(@F, x_init, l, u);
%            OR
%            [x_matrix, oscper, a_totsh, a_totsc, c_totsc ,a_numsh, ...
%            c_numsc] = mdim_sskw(@F, x_init, l, u ,[]);
 
% Author          : Mehmet Deniz Cicek
% Paper Reference : Multidimensional Applications of Scaled and Shifted
%                   Stochastic Approximation Algorithms
% Date            : 2009/12/01
% Version         : 3

global CRN

% Check if there are enough inputs
if nargin<4
    error('ScaledAndShiftedKW:AtLeastFourInputs',...
        'ScaledAndShiftedKW requires at least four input arguments.')
end

% Check if any options are defined; if not, use defaults
if nargin==4
    options=[];
end    
    
% Make F callable by feval
if ischar(F) & exist(F)~=2
   F = inline(F);
elseif isa(F,'sym')
   F = inline(char(F));
end

% Define the default options
defaultoptions = struct('m',10000,'n',15000,'theta_a',...
    1.0*ones(length(u),1),'a',1.0, 'theta_c',0.05*(u-l),'c',0.25, ...
    'c_scl',2.0, 'cmax',0.2*(u-l), 'a_sh_bd',10, 'a_numsc',4, ...
    'a_sc_bd',10,'a_maxnumsh',50, 'c_maxnumsc',50,'maxnumtrials',20,...
    'sampathno',1);
m=optimgetfast(options,'m',defaultoptions);
n=optimgetfast(options,'n',defaultoptions);
a=optimgetfast(options,'a',defaultoptions);
c=optimgetfast(options,'c',defaultoptions);
theta_a=optimgetfast(options,'theta_a',defaultoptions);
theta_c=optimgetfast(options,'theta_c',defaultoptions);
c_scl=optimgetfast(options,'c_scl',defaultoptions);
cmax=optimgetfast(options,'cmax',defaultoptions); 
a_sh_bd=optimgetfast(options,'a_sh_bd',defaultoptions);
a_sc_bd=optimgetfast(options,'a_sc_bd',defaultoptions);
a_numsc=optimgetfast(options,'a_numsc',defaultoptions);
a_maxnumsh=optimgetfast(options,'a_maxnumsh',defaultoptions);
c_maxnumsc=optimgetfast(options,'c_maxnumsc',defaultoptions);
maxnumtrials=optimgetfast(options,'maxnumtrials',defaultoptions);
sampathno=optimgetfast(options,'sampathno',defaultoptions);
defaultoptions2=struct('out_ind',m,'m_max',m);
out_ind=optimgetfast(options,'out_ind',defaultoptions2); 
m_max=optimgetfast(options,'m_max',defaultoptions2);

prec=eps(max(max(abs(u)),max(abs(l))));

[dim,s]=size(x_init);
% Set the x_init or n
if s==1
    x_init=x_init*ones(1,n);
else
    n=s;
end

% initialization
x_matrix = zeros(length(out_ind),dim,n);
oscpath=zeros(maxnumtrials*a_numsc,dim,n);
iternum=ones(1,n);
oscper   = a_numsc*ones(dim,n);
a_totsh  = zeros(dim,n);
x_vec    = x_init;
x_up     = x_vec;
x_dn     = x_vec;
c_vec    = theta_c*ones(1,n);
a_vec    = theta_a*ones(1,n);
a_vec_ind= ones(dim,n);
c_numsc  = zeros(dim,n);
a_numsh  = zeros(dim,n);
lastadapt= zeros(dim,n);
samplepath=zeros(length(sampathno),m-1,4*dim);
a_sh_bd  = a_sh_bd*ones(dim,n);
cmax=cmax*ones(1,n);
cmax_achieved=zeros(dim,n);
CRN=0;

j=1;
upper=u*ones(1,n);
lower=l*ones(1,n);
c1=c_vec;
ci_next=c1;
wasatboundary=(x_vec>=upper-c1-prec)-(x_vec<=lower+c1+prec);
x_vec=min(upper-c1,max(lower+c1,x_vec));
if (out_ind(1)==1)
        x_matrix(1, :, :) = x_vec;
        j=j+1;
end 
oscpath(1,:,:)=x_vec;

% The first for loop is for the iterations where the a_n sequence is 
% scaled up and there is no shifting    
for i = [ 1 : a_numsc ] 
    osc=zeros(dim,n); 
    numtrials=0;
    osccomp=zeros(1,n);
    while and(min(osccomp)==0,numtrials<=maxnumtrials)       
        numtrials=numtrials+1;
        ci=ci_next;
        ci_next=c_vec.*(ones(dim,1)*((iternum+1).^(-c)));
        wasatboundary=(x_vec>=upper-ci-prec)-(x_vec<=lower+ci+prec);
        x_vec_prev=x_vec;

        % Using common random numbers in finite difference estimate of the
        % gradient 
        CRN=0;
        for d=1:dim
            x_up = [x_vec(1:d-1,:); x_vec(d,:) + ci(d,:); x_vec(d+1:dim,:)];
            f_up=feval(F,x_up,varargin{:});
            CRN=1;
            x_dn = [x_vec(1:d-1,:); x_vec(d,:) - ci(d,:); x_vec(d+1:dim,:)];
            gradest(d,:)=(f_up-feval(F,x_dn,varargin{:}))./ci(d,:);
        end
        pushout=((sign(gradest).*wasatboundary)==1);
        x_vec=x_vec+gradest.*(a_vec.*(a_vec_ind.^(-a)));
        
        increase=(x_vec>=x_vec_prev);
        osc2=(1-osc).*(1-pushout);
        osc=max(osc,osc2);        
        truncfactor=((upper-ci_next).*increase+(lower+ci_next).*...
            (1-increase)-x_vec_prev)./gradest;
        scalefactor=min(max(1,truncfactor.*((a_vec_ind.^(a))./a_vec)),...
            a_sc_bd).*osc2+(1-osc2);
        a_vec=a_vec.*scalefactor;
        lastadapt=max(lastadapt,i*(scalefactor>1.0));
        
        % scale up c_n: We scale up only when it is large(small) and it 
        % was large(small) in the previous iteration as well and the  
        % maximum number of scale-ups is not exceeded. The scale-up factor  
        % should be calculated taking the upper bound on c_n into account.
        c_scaler=(ones(dim,1)*(1-osccomp)).*(c_numsc<c_maxnumsc).*...
            pushout.*(1-cmax_achieved);
        if (max(max(c_scaler))>0)
            c_multip=max(1.0,c_scaler.*min(c_scl,cmax.*...
                (ones(dim,1)*(iternum+1).^c)./c_vec));
            c_vec=c_vec.*c_multip;
            c_numsc=c_numsc+(c_multip>1.0);
            lastadapt=max(lastadapt,i*(c_multip>1.0));
            ci_next=c_vec.*(ones(dim,1)*((iternum+1).^(-c)));
            cmax_achieved=max(cmax_achieved,(ci_next>=cmax-prec));
        end
        x_vec=x_vec_prev.*(ones(dim,1)*osccomp)+...
            osc2.*(x_vec_prev + gradest.*truncfactor)+...
            (1-osc2-(ones(dim,1)*osccomp)).*x_vec;
        x_vec=min(upper-ci_next,max(lower+ci_next,x_vec));
        for t=1:length(sampathno)
            if osccomp(t)==0
                samplepath(t,iternum(t),1:dim)=gradest(:,sampathno(t))';
                samplepath(t,iternum(t),dim+1:2*dim)=...
                    (x_vec(:,sampathno(t)))';
                samplepath(t,iternum(t),2*dim+1:3*dim)=...
                    (a_vec(:,sampathno(t)).*((a_vec_ind(:,sampathno(t))+1).^(-a)))';
                samplepath(t,iternum(t),3*dim+1:4*dim)=...
                    (c_vec(:,sampathno(t)).*((iternum(t)+1).^(-c)))';
            end 
        end
        iternum=iternum+(1-osccomp);
        a_vec_ind=a_vec_ind+ones(dim,1)*(1-osccomp);
        osccomp=(sum(osc,1)==dim);
        
        itermtx=permute(repmat(iternum,[a_numsc*maxnumtrials 1 dim]),[1 3 2]);
        chgmtx=repmat([1:a_numsc*maxnumtrials]',[1 dim n]);
        chgmtx=(chgmtx==itermtx);
        oscpath=oscpath.*(1-chgmtx)+chgmtx.*...
            permute(repmat(x_vec,[1 1 a_numsc*maxnumtrials]),[3 1 2]);
        
    end        
    % end of while (sum(osc)<n)
    
    if numtrials==(maxnumtrials+1)
        percentosc=100*sum((sum(osc)==dim))/n;
        str=['Only ',num2str(percentosc),...
            '% of the paths completed the oscillation'];
        disp(str)
        pause
    end
end
% end of for i = [ 1 : a_numsc ]
oscfinish=iternum;
v=min(iternum);

while and(j<=length(out_ind),out_ind(min(j,length(out_ind)))<=min(iternum))
        x_matrix(j, :, :) = oscpath(out_ind(j),:,:);
        j=j+1;
end

% The second for loop is for the iterations where there is no scaling up 
% of the a_n sequence and we start shifting that sequence when necessary
for i = [ v : m-1 ]
    iterating=(ones(dim,1)*(iternum==i));
    ci=c_vec*(i^(-c));
    ci_next=c_vec*((i+1)^(-c));
    wasatboundary=(x_vec>=upper-ci-prec)-(x_vec<=lower+ci+prec);
    x_vec_prev=x_vec;
    CRN=0;
    for d=1:dim
        x_up = [x_vec(1:d-1,:); x_vec(d,:) + ci(d,:); x_vec(d+1:dim,:)];
        f_up=feval(F,x_up,varargin{:});
        CRN=1;
        x_dn = [x_vec(1:d-1,:); x_vec(d,:) - ci(d,:); x_vec(d+1:dim,:)];
        gradest(d,:)=(f_up-feval(F,x_dn,varargin{:}))./ci(d,:);
    end

    pushout=((sign(gradest).*wasatboundary)==1);
    x_vec = x_vec + gradest.*(a_vec.*(a_vec_ind.^(-a)));

    increase=(x_vec>=x_vec_prev);
    isatboundary=(x_vec>=upper-ci-prec)-(x_vec<=lower+ci+prec);
    
    % shift a_n: We shift the a_n sequence only if the iterate is
    % large(small) and it was small (large) before.    
    shiftind=(abs(isatboundary)==1).*(1-pushout).*...
        (a_numsh<a_maxnumsh).*(ones(dim,n)*(i<=m_max)).*...
        (abs(wasatboundary)==1).*iterating;
    
    if (max(max(shiftind))>0)
        lastadapt=max(lastadapt,i*shiftind);
        a_numsh=a_numsh+shiftind;
        truncfactor=((upper-ci_next).*increase+...
            (lower+ci_next).*(1-increase)-x_vec_prev)./gradest;
        shiftamount=shiftind.*min(a_sh_bd,max(a_vec_ind,...
            ceil((a_vec./truncfactor).^(1/a)))-a_vec_ind);
        a_vec_ind=a_vec_ind+shiftamount;
        lastadapt=max(lastadapt,i*shiftind);
        a_sh_bd=max(2*(shiftamount==a_sh_bd).*a_sh_bd,a_sh_bd);
    end

    %scale up c_n
    c_scaler=(c_numsc<c_maxnumsc).*(ones(dim,n)*(i<=m_max)).* ...
        pushout.*(1-cmax_achieved).*iterating;
    if(max(max(c_scaler))>0)
        c_multip=max(1.0,c_scaler.*min(c_scl,cmax*((i+1)^c)./c_vec));
        c_vec=c_vec.*c_multip;
        c_numsc=c_numsc+(c_multip>1.0);
        lastadapt=max(lastadapt,i*(c_multip>1.0));
        ci_next=c_vec*((i+1)^(-c));
        cmax_achieved=max(cmax_achieved,(ci_next>=cmax-prec));
    end
    
    x_vec=iterating.*min(upper-ci_next,max(lower+ci_next,x_vec))+...
        (1-iterating).*x_vec_prev;
    
    for t=1:length(sampathno)
        if iterating(1,t)==1
            samplepath(t,i,1:dim)=gradest(:,sampathno(t))';
            samplepath(t,i,dim+1:2*dim)=(x_vec(:,sampathno(t)))';
            samplepath(t,i,2*dim+1:3*dim)=(a_vec(:,sampathno(t)).*...
                ((a_vec_ind(:,sampathno(t))+1).^(-a)))';
            samplepath(t,i,3*dim+1:4*dim)=(c_vec(:,sampathno(t)).*...
                ((iternum(t)+1).^(-c)))';
        end
    end
    
    iternum=iternum+(iternum==i);
    oscper=max(oscper,i*(abs(isatboundary)==1).*...
        (abs(isatboundary-wasatboundary)>=2));
	a_vec_ind=a_vec_ind+1;
    if n==1
        x_vec2=iterating.*x_vec+(1-iterating).*...
            oscpath(min(i+1,max(oscfinish)),:,:)';
    else
        if dim ==1
            x_vec2=iterating.*x_vec+(1-iterating).*...
                squeeze(oscpath(min(i+1,max(oscfinish)),:,:))';
        else
            x_vec2=iterating.*x_vec+(1-iterating).*...
                squeeze(oscpath(min(i+1,max(oscfinish)),:,:));
        end
    end
    if j<=length(out_ind)
        if (out_ind(j)==i+1)
            x_matrix(j, :, :) = x_vec2;
            j=j+1;
        end
    end 
end
% end of for i = [ v : m-1 ]    

a_totsh=a_vec_ind-m;
a_totsc=a_vec./(theta_a*ones(1,n));
c_totsc=c_vec./(theta_c*ones(1,n));

% end of mdim_sskw.m
